"use strict";
// Copyright 2020 The Kubeflow Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var storage_1 = require("@google-cloud/storage");
var fs = __importStar(require("fs"));
var minio = __importStar(require("minio"));
var node_fetch_1 = __importDefault(require("node-fetch"));
var path = __importStar(require("path"));
var stream_1 = require("stream");
var supertest_1 = __importDefault(require("supertest"));
var app_1 = require("../app");
var configs_1 = require("../configs");
var serverInfo = __importStar(require("../helpers/server-info"));
var minioHelper = __importStar(require("../minio-helper"));
var test_helper_1 = require("./test-helper");
var MinioClient = minio.Client;
jest.mock('minio');
jest.mock('node-fetch');
jest.mock('@google-cloud/storage');
var mockedFetch = node_fetch_1.default;
describe('/artifacts', function () {
    var app;
    var argv = test_helper_1.commonSetup().argv;
    var artifactContent = 'hello world';
    beforeEach(function () {
        artifactContent = 'hello world'; // reset
        var mockedMinioClient = MinioClient;
        mockedMinioClient.mockImplementation(function () {
            var _this = this;
            return {
                getObject: function (bucket, key) { return __awaiter(_this, void 0, void 0, function () {
                    var objStream;
                    return __generator(this, function (_a) {
                        objStream = new stream_1.PassThrough();
                        objStream.end(artifactContent);
                        if (bucket === 'ml-pipeline' && key === 'hello/world.txt') {
                            return [2 /*return*/, objStream];
                        }
                        else {
                            throw new Error("Unable to retrieve " + bucket + "/" + key + " artifact.");
                        }
                        return [2 /*return*/];
                    });
                }); },
            };
        });
    });
    afterEach(function () {
        if (app) {
            app.close();
        }
    });
    describe('/get', function () {
        it('responds with a minio artifact if source=minio', function (done) {
            var mockedMinioClient = minio.Client;
            var configs = configs_1.loadConfigs(argv, {
                MINIO_ACCESS_KEY: 'minio',
                MINIO_HOST: 'minio-service',
                MINIO_NAMESPACE: 'kubeflow',
                MINIO_PORT: '9000',
                MINIO_SECRET_KEY: 'minio123',
                MINIO_SSL: 'false',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=minio&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent, function (err) {
                expect(mockedMinioClient).toBeCalledWith({
                    accessKey: 'minio',
                    endPoint: 'minio-service.kubeflow',
                    port: 9000,
                    secretKey: 'minio123',
                    useSSL: false,
                });
                done(err);
            });
        });
        it('responds with a s3 artifact if source=s3', function (done) {
            var mockedMinioClient = jest.spyOn(minioHelper, 'createMinioClient');
            var configs = configs_1.loadConfigs(argv, {
                AWS_ACCESS_KEY_ID: 'aws123',
                AWS_SECRET_ACCESS_KEY: 'awsSecret123',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=s3&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent, function (err) {
                expect(mockedMinioClient).toBeCalledWith({
                    accessKey: 'aws123',
                    endPoint: 's3.amazonaws.com',
                    region: 'us-east-1',
                    secretKey: 'awsSecret123',
                });
                done(err);
            });
        });
        it('responds with partial s3 artifact if peek=5 flag is set', function (done) {
            var mockedMinioClient = jest.spyOn(minioHelper, 'createMinioClient');
            var configs = configs_1.loadConfigs(argv, {
                AWS_ACCESS_KEY_ID: 'aws123',
                AWS_SECRET_ACCESS_KEY: 'awsSecret123',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=s3&bucket=ml-pipeline&key=hello%2Fworld.txt&peek=5')
                .expect(200, artifactContent.slice(0, 5), function (err) {
                expect(mockedMinioClient).toBeCalledWith({
                    accessKey: 'aws123',
                    endPoint: 's3.amazonaws.com',
                    region: 'us-east-1',
                    secretKey: 'awsSecret123',
                });
                done(err);
            });
        });
        it('responds with a s3 artifact from bucket in non-default region if source=s3', function (done) {
            var mockedMinioClient = jest.spyOn(minioHelper, 'createMinioClient');
            var configs = configs_1.loadConfigs(argv, {
                AWS_ACCESS_KEY_ID: 'aws123',
                AWS_SECRET_ACCESS_KEY: 'awsSecret123',
                AWS_REGION: 'eu-central-1',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=s3&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent, function (err) {
                expect(mockedMinioClient).toBeCalledWith({
                    accessKey: 'aws123',
                    endPoint: 's3.amazonaws.com',
                    region: 'eu-central-1',
                    secretKey: 'awsSecret123',
                });
                done(err);
            });
        });
        it('responds with a http artifact if source=http', function (done) {
            var artifactContent = 'hello world';
            mockedFetch.mockImplementationOnce(function (url, opts) {
                return url === 'http://foo.bar/ml-pipeline/hello/world.txt'
                    ? Promise.resolve({
                        buffer: function () { return Promise.resolve(artifactContent); },
                        body: new stream_1.PassThrough().end(artifactContent),
                    })
                    : Promise.reject('Unable to retrieve http artifact.');
            });
            var configs = configs_1.loadConfigs(argv, {
                HTTP_BASE_URL: 'foo.bar/',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=http&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent, function (err) {
                expect(mockedFetch).toBeCalledWith('http://foo.bar/ml-pipeline/hello/world.txt', {
                    headers: {},
                });
                done(err);
            });
        });
        it('responds with partial http artifact if peek=5 flag is set', function (done) {
            var artifactContent = 'hello world';
            var mockedFetch = node_fetch_1.default;
            mockedFetch.mockImplementationOnce(function (url, opts) {
                return url === 'http://foo.bar/ml-pipeline/hello/world.txt'
                    ? Promise.resolve({
                        buffer: function () { return Promise.resolve(artifactContent); },
                        body: new stream_1.PassThrough().end(artifactContent),
                    })
                    : Promise.reject('Unable to retrieve http artifact.');
            });
            var configs = configs_1.loadConfigs(argv, {
                HTTP_BASE_URL: 'foo.bar/',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=http&bucket=ml-pipeline&key=hello%2Fworld.txt&peek=5')
                .expect(200, artifactContent.slice(0, 5), function (err) {
                expect(mockedFetch).toBeCalledWith('http://foo.bar/ml-pipeline/hello/world.txt', {
                    headers: {},
                });
                done(err);
            });
        });
        it('responds with a https artifact if source=https', function (done) {
            var artifactContent = 'hello world';
            mockedFetch.mockImplementationOnce(function (url, opts) {
                return url === 'https://foo.bar/ml-pipeline/hello/world.txt' &&
                    opts.headers.Authorization === 'someToken'
                    ? Promise.resolve({
                        buffer: function () { return Promise.resolve(artifactContent); },
                        body: new stream_1.PassThrough().end(artifactContent),
                    })
                    : Promise.reject('Unable to retrieve http artifact.');
            });
            var configs = configs_1.loadConfigs(argv, {
                HTTP_AUTHORIZATION_DEFAULT_VALUE: 'someToken',
                HTTP_AUTHORIZATION_KEY: 'Authorization',
                HTTP_BASE_URL: 'foo.bar/',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=https&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent, function (err) {
                expect(mockedFetch).toBeCalledWith('https://foo.bar/ml-pipeline/hello/world.txt', {
                    headers: {
                        Authorization: 'someToken',
                    },
                });
                done(err);
            });
        });
        it('responds with a https artifact using the inherited header if source=https and http authorization key is provided.', function (done) {
            var artifactContent = 'hello world';
            mockedFetch.mockImplementationOnce(function (url, _opts) {
                return url === 'https://foo.bar/ml-pipeline/hello/world.txt'
                    ? Promise.resolve({
                        buffer: function () { return Promise.resolve(artifactContent); },
                        body: new stream_1.PassThrough().end(artifactContent),
                    })
                    : Promise.reject('Unable to retrieve http artifact.');
            });
            var configs = configs_1.loadConfigs(argv, {
                HTTP_AUTHORIZATION_KEY: 'Authorization',
                HTTP_BASE_URL: 'foo.bar/',
            });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=https&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .set('Authorization', 'inheritedToken')
                .expect(200, artifactContent, function (err) {
                expect(mockedFetch).toBeCalledWith('https://foo.bar/ml-pipeline/hello/world.txt', {
                    headers: {
                        Authorization: 'inheritedToken',
                    },
                });
                done(err);
            });
        });
        it('responds with a gcs artifact if source=gcs', function (done) {
            var artifactContent = 'hello world';
            var mockedGcsStorage = storage_1.Storage;
            var stream = new stream_1.PassThrough();
            stream.write(artifactContent);
            stream.end();
            mockedGcsStorage.mockImplementationOnce(function () { return ({
                bucket: function () { return ({
                    getFiles: function () {
                        return Promise.resolve([[{ name: 'hello/world.txt', createReadStream: function () { return stream; } }]]);
                    },
                }); },
            }); });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=gcs&bucket=ml-pipeline&key=hello%2Fworld.txt')
                .expect(200, artifactContent + '\n', done);
        });
        it('responds with a partial gcs artifact if peek=5 is set', function (done) {
            var artifactContent = 'hello world';
            var mockedGcsStorage = storage_1.Storage;
            var stream = new stream_1.PassThrough();
            stream.end(artifactContent);
            mockedGcsStorage.mockImplementationOnce(function () { return ({
                bucket: function () { return ({
                    getFiles: function () {
                        return Promise.resolve([[{ name: 'hello/world.txt', createReadStream: function () { return stream; } }]]);
                    },
                }); },
            }); });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=gcs&bucket=ml-pipeline&key=hello%2Fworld.txt&peek=5')
                .expect(200, artifactContent.slice(0, 5), done);
        });
        it('responds with a volume artifact if source=volume', function (done) {
            var artifactContent = 'hello world';
            var tempPath = path.join(test_helper_1.mkTempDir(), 'content');
            fs.writeFileSync(tempPath, artifactContent);
            jest.spyOn(serverInfo, 'getHostPod').mockImplementation(function () {
                return Promise.resolve([
                    {
                        spec: {
                            containers: [
                                {
                                    volumeMounts: [
                                        {
                                            name: 'artifact',
                                            mountPath: path.dirname(tempPath),
                                            subPath: 'subartifact',
                                        },
                                    ],
                                    name: 'ml-pipeline-ui',
                                },
                            ],
                            volumes: [
                                {
                                    name: 'artifact',
                                    persistentVolumeClaim: {
                                        claimName: 'artifact_pvc',
                                    },
                                },
                            ],
                        },
                    },
                    undefined,
                ]);
            });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/get?source=volume&bucket=artifact&key=subartifact/content')
                .expect(200, artifactContent, done);
        });
        it('responds with a partial volume artifact if peek=5 is set', function (done) {
            var artifactContent = 'hello world';
            var tempPath = path.join(test_helper_1.mkTempDir(), 'content');
            fs.writeFileSync(tempPath, artifactContent);
            jest.spyOn(serverInfo, 'getHostPod').mockImplementation(function () {
                return Promise.resolve([
                    {
                        spec: {
                            containers: [
                                {
                                    volumeMounts: [
                                        {
                                            name: 'artifact',
                                            mountPath: path.dirname(tempPath),
                                        },
                                    ],
                                    name: 'ml-pipeline-ui',
                                },
                            ],
                            volumes: [
                                {
                                    name: 'artifact',
                                    persistentVolumeClaim: {
                                        claimName: 'artifact_pvc',
                                    },
                                },
                            ],
                        },
                    },
                    undefined,
                ]);
            });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get("/artifacts/get?source=volume&bucket=artifact&key=content&peek=5")
                .expect(200, artifactContent.slice(0, 5), done);
        });
        it('responds error with a not exist volume', function (done) {
            jest.spyOn(serverInfo, 'getHostPod').mockImplementation(function () {
                return Promise.resolve([
                    {
                        metadata: {
                            name: 'ml-pipeline-ui',
                        },
                        spec: {
                            containers: [
                                {
                                    volumeMounts: [
                                        {
                                            name: 'artifact',
                                            mountPath: '/foo/bar/path',
                                        },
                                    ],
                                    name: 'ml-pipeline-ui',
                                },
                            ],
                            volumes: [
                                {
                                    name: 'artifact',
                                    persistentVolumeClaim: {
                                        claimName: 'artifact_pvc',
                                    },
                                },
                            ],
                        },
                    },
                    undefined,
                ]);
            });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get("/artifacts/get?source=volume&bucket=notexist&key=content")
                .expect(404, 'Failed to open volume://notexist/content, Cannot find file "volume://notexist/content" in pod "ml-pipeline-ui": volume "notexist" not configured', done);
        });
        it('responds error with a not exist volume mount path if source=volume', function (done) {
            jest.spyOn(serverInfo, 'getHostPod').mockImplementation(function () {
                return Promise.resolve([
                    {
                        metadata: {
                            name: 'ml-pipeline-ui',
                        },
                        spec: {
                            containers: [
                                {
                                    volumeMounts: [
                                        {
                                            name: 'artifact',
                                            mountPath: '/foo/bar/path',
                                            subPath: 'subartifact',
                                        },
                                    ],
                                    name: 'ml-pipeline-ui',
                                },
                            ],
                            volumes: [
                                {
                                    name: 'artifact',
                                    persistentVolumeClaim: {
                                        claimName: 'artifact_pvc',
                                    },
                                },
                            ],
                        },
                    },
                    undefined,
                ]);
            });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get("/artifacts/get?source=volume&bucket=artifact&key=notexist/config")
                .expect(404, 'Failed to open volume://artifact/notexist/config, Cannot find file "volume://artifact/notexist/config" in pod "ml-pipeline-ui": volume "artifact" not mounted or volume "artifact" with subPath (which is prefix of notexist/config) not mounted', done);
        });
        it('responds error with a not exist volume mount artifact if source=volume', function (done) {
            jest.spyOn(serverInfo, 'getHostPod').mockImplementation(function () {
                return Promise.resolve([
                    {
                        spec: {
                            containers: [
                                {
                                    volumeMounts: [
                                        {
                                            name: 'artifact',
                                            mountPath: '/foo/bar',
                                            subPath: 'subartifact',
                                        },
                                    ],
                                    name: 'ml-pipeline-ui',
                                },
                            ],
                            volumes: [
                                {
                                    name: 'artifact',
                                    persistentVolumeClaim: {
                                        claimName: 'artifact_pvc',
                                    },
                                },
                            ],
                        },
                    },
                    undefined,
                ]);
            });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get("/artifacts/get?source=volume&bucket=artifact&key=subartifact/notxist.csv")
                .expect(500, "Failed to open volume://artifact/subartifact/notxist.csv: Error: ENOENT: no such file or directory, stat '/foo/bar/notxist.csv'", done);
        });
    });
    describe('/:source/:bucket/:key', function () {
        it('downloads a minio artifact', function (done) {
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/minio/ml-pipeline/hello/world.txt') // url
                .expect(200, artifactContent, done);
        });
        it('downloads a tar gzipped artifact as is', function (done) {
            // base64 encoding for tar gzipped 'hello-world'
            var tarGzBase64 = 'H4sIAFa7DV4AA+3PSwrCMBRG4Y5dxV1BuSGPridgwcItkTZSl++johNBJ0WE803OIHfwZ87j0fq2nmuzGVVNIcitXYqPpntXLojzSb33MToVdTG5rhHdbtLLaa55uk5ZBrMhj23ty9u7T+/rT+TZP3HozYosZbL97tdbAAAAAAAAAAAAAAAAAADfuwAyiYcHACgAAA==';
            var tarGzBuffer = Buffer.from(tarGzBase64, 'base64');
            artifactContent = tarGzBuffer;
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/artifacts/minio/ml-pipeline/hello/world.txt') // url
                .expect(200, tarGzBuffer.toString(), done);
        });
    });
});
//# sourceMappingURL=artifact-get.test.js.map