"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright 2020 The Kubeflow Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var stream_1 = require("stream");
var utils_1 = require("./utils");
describe('utils', function () {
    describe('PreviewStream', function () {
        it('should stream first 5 bytes', function (done) {
            var peek = 5;
            var input = 'some string that will be truncated.';
            var source = new stream_1.PassThrough();
            var preview = new utils_1.PreviewStream({ peek: peek });
            var dst = source.pipe(preview).on('end', done);
            source.end(input);
            dst.once('readable', function () { return expect(dst.read().toString()).toBe(input.slice(0, peek)); });
        });
        it('should stream everything if peek==0', function (done) {
            var peek = 0;
            var input = 'some string that will be truncated.';
            var source = new stream_1.PassThrough();
            var preview = new utils_1.PreviewStream({ peek: peek });
            var dst = source.pipe(preview).on('end', done);
            source.end(input);
            dst.once('readable', function () { return expect(dst.read().toString()).toBe(input); });
        });
    });
    describe('findFileOnPodVolume', function () {
        var podTemplateSpec = {
            spec: {
                containers: [
                    {
                        volumeMounts: [
                            {
                                name: 'output',
                                mountPath: '/main',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline1',
                                mountPath: '/main1',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline2',
                                mountPath: '/main2',
                            },
                        ],
                        name: 'main',
                    },
                    {
                        volumeMounts: [
                            {
                                name: 'output',
                                mountPath: '/data',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline1',
                                mountPath: '/data1',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline2',
                                mountPath: '/data2',
                            },
                        ],
                        name: 'ml-pipeline-ui',
                    },
                ],
                volumes: [
                    {
                        name: 'output',
                        hostPath: {
                            path: '/data/output',
                            type: 'Directory',
                        },
                    },
                    {
                        name: 'artifact',
                        persistentVolumeClaim: {
                            claimName: 'artifact_pvc',
                        },
                    },
                ],
            },
        };
        it('parse file path with containerNames', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: ['ml-pipeline-ui', 'ml-pipeline-ui-artifact'],
                volumeMountName: 'output',
                filePathInVolume: 'a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual(undefined);
            expect(filePath).toEqual('/data/a/b/c');
        });
        it('parse file path with containerNames and subPath', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: ['ml-pipeline-ui', 'ml-pipeline-ui-artifact'],
                volumeMountName: 'artifact',
                filePathInVolume: 'pipeline1/a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual(undefined);
            expect(filePath).toEqual('/data1/a/b/c');
        });
        it('parse file path without containerNames', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: undefined,
                volumeMountName: 'output',
                filePathInVolume: 'a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual(undefined);
            expect(filePath).toEqual('/main/a/b/c');
        });
        it('parse file path error with not exist volume', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: undefined,
                volumeMountName: 'other',
                filePathInVolume: 'a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual('Cannot find file "volume://other/a/b/c" in pod "unknown": volume "other" not configured');
            expect(filePath).toEqual('');
        });
        it('parse file path error with not exist container', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: ['other1', 'other2'],
                volumeMountName: 'output',
                filePathInVolume: 'a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual('Cannot find file "volume://output/a/b/c" in pod "unknown": container "other1" or "other2" not found');
            expect(filePath).toEqual('');
        });
        it('parse file path error with volume not mount error', function () {
            var _a = __read(utils_1.findFileOnPodVolume(podTemplateSpec, {
                containerNames: undefined,
                volumeMountName: 'artifact',
                filePathInVolume: 'a/b/c',
            }), 2), filePath = _a[0], err = _a[1];
            expect(err).toEqual('Cannot find file "volume://artifact/a/b/c" in pod "unknown": volume "artifact" not mounted or volume "artifact" with subPath (which is prefix of a/b/c) not mounted');
            expect(filePath).toEqual('');
        });
    });
    describe('resolveFilePathOnVolume', function () {
        it('undefined volumeMountSubPath', function () {
            var path = utils_1.resolveFilePathOnVolume({
                filePathInVolume: 'a/b/c',
                volumeMountPath: '/data',
                volumeMountSubPath: undefined,
            });
            expect(path).toEqual(['/data/a/b/c', undefined]);
        });
        it('with volumeMountSubPath', function () {
            var path = utils_1.resolveFilePathOnVolume({
                volumeMountPath: '/data',
                filePathInVolume: 'a/b/c',
                volumeMountSubPath: 'a',
            });
            expect(path).toEqual(['/data/b/c', undefined]);
        });
        it('with multiple layer volumeMountSubPath', function () {
            var path = utils_1.resolveFilePathOnVolume({
                volumeMountPath: '/data',
                filePathInVolume: 'a/b/c',
                volumeMountSubPath: 'a/b',
            });
            expect(path).toEqual(['/data/c', undefined]);
        });
        it('with not exist volumeMountSubPath', function () {
            var path = utils_1.resolveFilePathOnVolume({
                volumeMountPath: '/data',
                filePathInVolume: 'a/b/c',
                volumeMountSubPath: 'other',
            });
            expect(path).toEqual([
                '',
                'File a/b/c not mounted, expecting the file to be inside volume mount subpath other',
            ]);
        });
    });
});
//# sourceMappingURL=utils.test.js.map